from setuptools import setup, find_packages

setup(
    name = "spyd",
    version = "0.1.0",
    packages = find_packages(),
    install_requires = ["pyyaml"],
    author = "Mazurex",
    author_email = "business.mazurex@gmail.com",
    description = "A smarter python dictionary",
    long_description = open("README.md").read(),
    long_description_content_type = "text/markdown",
    url = "https://github.com/Mazurex/SPYD",
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independant"
    ],
    python_requires = ">=3.7",
)